#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/colour_methods/colour_methods.rb'
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/revert.rb'
  require 'colours/colour_table/colour_table.rb'
  require 'colours/map_symbol_to_corresponding_colour/map_symbol_to_corresponding_colour.rb'
  require 'colours/autogenerated/support_for_html_colours.rb'

  # ========================================================================= #
  # === Colours.ecomment
  #
  # This method was added on Nov 2013.
  #
  # In April 2014, the ability to split at any arbitrary character was
  # added, which can be controlled via the second argument given to
  # that method. By default it will split on the token '#'.
  # ========================================================================= #
  def self.ecomment(
      i,
      optional_split_at_this_character = '#', # ← Specify which character to split at.
      first_colour_to_use         = :steelblue,
      second_colour_to_use        = :seagreen,
      colour_to_use_for_the_token = :slategray
    )
    _ = i.to_s
    if optional_split_at_this_character.is_a? Hash
      # ===================================================================== #
      # === :token
      # ===================================================================== #
      if optional_split_at_this_character.has_key? :token
        optional_split_at_this_character = optional_split_at_this_character.delete :token
      end
    end
    optional_split_at_this_character = optional_split_at_this_character.to_s
    # ======================================================================= #
    # === If we can find a token to split at
    # ======================================================================= #
    if _.include? optional_split_at_this_character
      _ = _.dup if _.frozen?
      splitted = _.split(optional_split_at_this_character)
      first_part  = ::Colours::SupportForHTMLColours.send(first_colour_to_use, splitted.first)
      middle_part = ::Colours::SupportForHTMLColours.send(colour_to_use_for_the_token, optional_split_at_this_character)
      second_part = ::Colours::SupportForHTMLColours.send(second_colour_to_use, splitted.last)
      _ = first_part+
          middle_part+
          second_part+
          ::Colours.rev
    end
    e _
  end

  # ========================================================================= #
  # === Colours.default_colour
  #
  # Invocation example:
  #
  #   puts Colours.default_colour('Hello world!')
  #
  # ========================================================================= #
  def self.default_colour(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if make_newline
        i = "#{i}#{N}"
      end
      if @use_html_colours
        i = SupportForHTMLColours.send(@colour_table[__method__.to_sym], i)
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])  
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    return i
  end

  # ========================================================================= #
  # === edir
  # ========================================================================= #
  def edefault_colour(
      i            = '',
      make_newline = false
    )
    e ::Colours.default_colour(i, make_newline)
  end

  # ========================================================================= #
  # === default_colour
  # ========================================================================= #
  def default_colour(i = '')
    ::Colours.default_colour(i)
  end

  # ========================================================================= #
  # === Colours.sdir
  #
  # Invocation example:
  #
  #   puts Colours.sdir('Hello world!')
  #
  # ========================================================================= #
  def self.sdir(
      i            = '',
      make_newline = false
    )
    if make_newline
      i = "#{i}#{N}"
    end
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(@colour_table[__method__.to_sym], i)
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])  
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end

  # ========================================================================= #
  # === edir
  # ========================================================================= #
  def edir(
      i            = '',
      make_newline = false
    )
    e ::Colours.sdir(i, make_newline)
  end

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i = '')
    ::Colours::sdir(i)
  end

  # ========================================================================= #
  # === Colours.sargument
  #
  # Invocation example:
  #
  #   Colours.sargument('Hello world!')
  #
  # ========================================================================= #
  def self.sargument(
      i            = '',
      make_newline = false
    )
    if make_newline
      i = "#{i}#{N}"
    end
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(@colour_table[__method__.to_sym], i)
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end; self.instance_eval { alias argument sargument } # === Colours.argument
       self.instance_eval { alias sarg     sargument } # === Colours.sarg

  # ========================================================================= #
  # === sargument
  # ========================================================================= #
  def sargument(
      i = '',
      make_newline = false
    )
    ::Colours.sargument(i, make_newline)
  end; alias sarg sargument # === sarg
  
  # ========================================================================= #
  # === eargument
  # 
  # The various simple colourize methods.
  # ========================================================================= #
  def eargument(
      i            = '',
      make_newline = false
    )
    e ::Colours.sargument(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.sfile
  #
  # This method can be explicitely used to colour files, such as
  # '/opt/foobar.rb'.
  #
  # Invocation examples:
  #
  #   Colours.sfile('/opt/foobar.rb')
  #   Colours.sfile('/tmp/test.md')
  #   Colours.sfile('Hello world!')
  #
  # ========================================================================= #
  def self.sfile(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    ::Colours.sfile(i)
  end

  # ========================================================================= #
  # === efile
  # ========================================================================= #
  def efile(
      i            = '',
      make_newline = false
    )
    e sfile(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.snormal
  #
  # Invocation example:
  #
  #   Colours.snormal('Hello world!')
  #
  # ========================================================================= #
  def self.snormal(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end

  # ========================================================================= #
  # === snormal
  # ========================================================================= #
  def snormal(
      i            = '',
      make_newline = false
    )
    ::Colours.snormal(i, make_newline)
  end

  # ========================================================================= #
  # === enormal
  # ========================================================================= #
  def enormal(
      i            = '',
      make_newline = false
    )
    e snormal(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.simportant
  #
  #This method is the one to denote "important" text components.
  #
  # Invocation examples:
  #
  #   Colours.simportant('Hello world!')
  #   Colours.simportant('yo there')
  #
  # ========================================================================= #
  def self.simportant(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end; self.instance_eval { alias simp simportant } # === Colours.simp

  # ========================================================================= #
  # === simportant
  #
  # Delegate towards Colours.simportant() here.
  # ========================================================================= #
  def simportant(
      i            = '',
      make_newline = false
    )
    ::Colours.simportant(i, make_newline)
  end; alias simp simportant # === simp
       alias si   simportant # === si

  # ========================================================================= #
  # === ciif
  # ========================================================================= #
  def ciif(i)
    eimportant(i, false)
  end # Alias to the above, basically.

  # ========================================================================= #
  # === Colours.eimp
  # ========================================================================= #
  def self.eimp(
      i            = '',
      make_newline = false
    )
    e simp(i, make_newline)
  end

  # ========================================================================= #
  # === eimportant
  # ========================================================================= #
  def eimportant(
      i            = '',
      make_newline = false
    )
    ::Colours.eimp(i, make_newline)
  end; alias eimp eimportant # === eimp
       alias cii  eimportant # === cii

  # ========================================================================= #
  # === Colours.swarn
  #
  # Invocation example:
  #
  #   Colours.swarn('Hello world!')
  #
  # ========================================================================= #
  def self.swarn(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(
      i            = '',
      make_newline = false
    )
    ::Colours.swarn(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.ewarn
  # ========================================================================= #
  def self.ewarn(
      i            = '',
      make_newline = false
    )
    e swarn(i, make_newline)
  end

  # ========================================================================= #
  # === ewarn
  # ========================================================================= #
  def ewarn(
      i            = '',
      make_newline = false
    )
    ::Colours.ewarn(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.ssymlink
  #
  # Invocation example:
  #
  #   Colours.ssymlink('Hello world!')
  #
  # ========================================================================= #
  def self.ssymlink(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end; self.instance_eval { alias sym  ssymlink } # === Colours.sym
       self.instance_eval { alias ssym ssymlink } # === Colours.ssym

  # ========================================================================= #
  # === Colours.colour_for_symlinks
  #
  # This method will simply return the colour for symlinks.
  # ========================================================================= #
  def self.colour_for_symlinks
    result = ''.dup
    if @use_colours
      if @use_html_colours
        result = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[:ssymlink])
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        result = SupportFor256Colours.send(use_this_colour)
      else
        result = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"
      end
    end
    result
  end

  # ========================================================================= #
  # === ssymlink
  #
  # Note that a symlink called symlink() exists to this method, but it
  # is not exactly clear whether this alias will be kept, as it may
  # interfere with some other methods when we do an include-action.
  # ========================================================================= #
  def ssymlink(i = '')
    ::Colours.ssymlink(i)
  end; alias symlink ssymlink # === symlink
       alias ssym    ssymlink # === ssym
       alias slink   ssymlink # === slink

  # ========================================================================= #
  # === Colours.esymlink
  # ========================================================================= #
  def self.esymlink(
      i            = '',
      make_newline = false
    )
    e ::Colours.ssymlink(i, make_newline)
  end

  # ========================================================================= #
  # === esymlink
  # ========================================================================= #
  def esymlink(
      i            = '',
      make_newline = false
    )
    ::Colours.esymlink(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.sfancy
  #
  # Invocation example:
  #
  #   Colours.sfancy('Hello world!')
  #
  # ========================================================================= #
  def self.sfancy(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(
          map_this_symbol_to_that_html_colour(@colour_table[__method__.to_sym]), i
        )
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end

  # ========================================================================= #
  # === Colours.efancy
  #
  # This simply will output the result from sfancy(), defined above.
  # ========================================================================= #
  def self.efancy(
      i            = '',
      make_newline = false
    )
    e sfancy(i, make_newline)
  end

  # ========================================================================= #
  # === efancy
  # ========================================================================= #
  def efancy(
      i            = '',
      make_newline = false
    )
    ::Colours.efancy(i, make_newline)
  end; alias f efancy # === f (f tag)

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    ::Colours.sfancy(i)
  end

  # ========================================================================= #
  # === cif
  #
  # This is efancy, but we won't use newlines.
  # ========================================================================= #
  def cif(i)
    efancy(i, false)
  end; alias ff cif # === ff (alias to the above ^^^)

  # ========================================================================= #
  # === Colours.scomments
  #
  # Invocation example:
  #
  #   Colours.scomments('Hello world!')
  #
  # ========================================================================= #
  def self.scomments(
      i            = '',
      make_newline = false
    )
    if @use_colours
      if @use_html_colours
        i = SupportForHTMLColours.send(@colour_table[__method__.to_sym], i)
      elsif @use_256_colours
        use_this_colour = map_this_symbol_to_that_256_colour(@colour_table[__method__.to_sym])
        i = SupportFor256Colours.send(use_this_colour, i)
      else
        i = "#{corresponding_colour?(@colour_table[__method__.to_sym])}"\
            "#{i}"\
            "#{rev}"
      end
    end
    i
  end; self.instance_eval { alias comment scomments } # === Colours.comment

  # ========================================================================= #
  # === scomments
  # ========================================================================= #
  def scomments(i = '')
    ::Colours.scomments(i) # Delegate towards Colours.scomments? here.
  end; alias scomment scomments # === scomment

  # ========================================================================= #
  # === Colours.scomments?
  #
  # This method will usually return :grey.
  # ========================================================================= #
  def self.scomments?
    colour_table?[:scomments]
  end

  # ========================================================================= #
  # === ecomment
  #
  # Just a wrapper over the Colours.ecomment() functionality.
  # ========================================================================= #
  def ecomment(
      i,
      optional_split_at_this_character = '#'
    )
    ::Colours.ecomment(i, optional_split_at_this_character)
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e ::Colours.sargument('Hello world!')
  e ::Colours.sfile('Hello world!')+' And this should appear in another colour.'
  e ::Colours.snormal('Hello world!')
  e ::Colours.simportant('Hello world!')
  e ::Colours.swarn('Hello world!')
  e ::Colours.ssymlink('Hello world!')
  e ::Colours.sfancy('Hello world!')
  e ::Colours.default_colour('Hello world!')
  e ::Colours.sdir('Hello world!')
  e ::Colours.scomments('Hello world!')
  ::Colours.ecomment('Hello world! # this is the associated comment')
end