#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/commandline/menu.rb'
# =========================================================================== #
module Colours

  require 'colours/html_colours/html_colours.rb'
  require 'colours/toplevel_methods/show_basic_colour_palette.rb'

  # ========================================================================= #
  # === Colours.menu                                               (menu tag)
  #
  # To invoke this method, try:
  #
  #   colours --show_html_colours
  #   colours file?
  #
  # ========================================================================= #
  def self.menu(
      i = ::Colours.commandline?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === colours --version
      #
      # Show the current version of the colours gem.
      # ===================================================================== #
      when /^-?-?versions?\??$/i
        show_the_version
      # ===================================================================== #
      # === colours --generate_shell_file_containing_the_html_colours
      #
      # This entry point will autogenerate the shell-file that contains
      # the HTML colours (RGB colours).
      # ===================================================================== #
      when /^-?-?generate(-|_)?shell(-|_)?file(-|_)?containing(-|_)?the(-|_)?html(-|_)?colours$/i,
           /^-?-?create(-|_)?shell(-|_)?file$/i # === colours --create-shell-file
        generate_shell_file_containing_the_html_colours
      # ===================================================================== #
      # === colours --show-html_colours
      # ===================================================================== #
      when /^-?-?show(-|_)?html(-|_)?colours$/i,
           /^-?-?show(-|_)?all(-|_)?html(-|_)?colours$/i,
           /^-?-?show(-|_)?all(-|_)?colours$/i,
           /^-?-?html(-|_)?colours$/i,
           'hcolours',
           'scolours',
           'all',
           'ALL',
           '1',
           'ALL_COLOURS',
           'colours?',
           'all_colours',
           'COLOURS',
           'html',
           '' # Show all colours here
        ::Colours.show_html_colours
        exit
      # ===================================================================== #
      # === colours --basics
      # ===================================================================== #
      when /^-?-?basics?$/i
        ::Colours.show_basic_colour_palette
        exit
      # ===================================================================== #
      # === colours --open
      # ===================================================================== #
      when 'OPEN',
           /^-?-?open$/
        open_this_file :then_exit
      # ===================================================================== #
      # === colours --file?
      # ===================================================================== #
      when /^-?-?file\??$/i,
           'file','file?'
        cliner {
          e 'We will next read in from the default file, from:'
          e
          e "  #{sfile(TEST_FILE)}"
          e
          read_and_display_this_file
        }
      # ===================================================================== #
      # === colours --project_base_directory?
      # ===================================================================== #
      when /^-?-?project(_|-)?base(_|-)?dir\??$/i,
           /^-?-?project(_|-)?base(_|-)?directory\??$/i,
           /^-?-?base(_|-)?dir\??$/i
        e PROJECT_BASE_DIRECTORY
      # ===================================================================== #
      # === colours --show-colour-palette
      # === colours --palette
      #
      # This entry point will show the basic colour palette.
      # ===================================================================== #
      when /^-?-?show(-|_)?colou?r(-|_)?palette$/i,
           /^-?-?show(-|_)?palette$/i,
           /^-?-?palette$/i
        show_colour_palette
      # ===================================================================== #
      # === colours --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
      end
    end
  end; self.instance_eval { alias check_against_menu menu } # === Colours.check_against_menu
                                                            # Add an alias to the above. ^^^

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i = nil)
    ::Colours.menu(i) # Delegate towards the class method above here.
  end

end