#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/html_colours/html_colours.rb'
# =========================================================================== #
module Colours

module HtmlColours # === Colours::HtmlColours

  require 'yaml'
  require 'colours/constants/file_constants.rb'
  require 'colours/toplevel_methods/e.rb'
  require 'colours/toplevel_methods/revert.rb'

  # ========================================================================= #
  # === Colours::HtmlColours.rev
  # ========================================================================= #
  def self.rev
    ::Colours.rev
  end

  # =========================================================================== #
  # === Colours::HtmlColours::HASH_HTML_COLOURS
  #
  # The colours in the yaml file are stored in a:
  #
  #     name: hexvalue
  #
  # format.
  # =========================================================================== #
  _ = FILE_HTML_COLOURS

  # =========================================================================== #
  # === @hash_html_colours
  # =========================================================================== #
  @hash_html_colours = {}

  if File.exist? _
    @hash_html_colours = YAML.load_file(_)
  else
    # ======================================================================= #
    # The second check is not ideal but I think that I only need this
    # on my home system anyway.
    # ======================================================================= #
    if ("#{ENV['IS_ROEBE']}" == '1')
      puts "Colours::HtmlColours: We could not find any "\
           "file at `#{_}`."
    end
  end

  # =========================================================================== #
  # === Colours::HtmlColours.hash_html_colours?
  # =========================================================================== #
  def self.hash_html_colours?
    @hash_html_colours
  end; self.instance_eval { alias main_hash? hash_html_colours? } # === Colours::HtmlColours.main_hash?

  # ========================================================================= #
  # === Colours::HtmlColours.show_html_colours
  #
  # This can be used to output the colours. It requires the method
  # html_colours().
  #
  # To invoke this method from the commandline, try:
  #
  #   colours --show_html_colours
  #
  # ========================================================================= #
  def self.show_html_colours
    require 'colours/rgb/rgb.rb'
    lpad = 22
    show_this_standard_sentence =
      'This is a longer test-sentence in that colour.'
    print '  '; e '=' * 100
    html_colours.each {|this_html_colour|
      result = "#{::Colours.rgb_as_string(this_html_colour)} #{this_html_colour.to_s.ljust(22)}"
      # ===================================================================== #
      # Next, show that particular colour.
      # ===================================================================== #
      string = ::Colours.rgb_as_string(this_html_colour).strip
      ansi_escape_code = string.tr("\e","\\e")
      display_this_line =
        "  | #{(this_html_colour+': ').ljust(lpad)}| "\
        "#{result}#{::Colours.rev} | #{::Colours.rgb_as_string(this_html_colour)}#{show_this_standard_sentence}#{rev} |\n".dup # rev() is needed to restore the old default.
      display_this_line << '  |                       | '
      display_this_line << ''.ljust(22)
      echo_string = "  |   echo -e \""+string+"\\"+ansi_escape_code+"xyz#{rev}"+'"'
      padded_echo_string = (echo_string.rstrip).ljust(72)
      display_this_line << "#{padded_echo_string}|"
      e display_this_line
    }
    print '  '; e '=' * 100
  end; self.instance_eval { alias display_all_colours show_html_colours } # === Colours::HtmlColours.display_all_colours
       self.instance_eval { alias show_colours        show_html_colours } # === Colours::HtmlColours.show_colours
       self.instance_eval { alias test                show_html_colours } # === Colours::HtmlColours.test
       self.instance_eval { alias show_all_colours    show_html_colours } # === Colours::HtmlColours.show_all_colours

  # =========================================================================== #
  # === Colours::HtmlColours.does_include?
  #
  # This method will return true if the given input is included in the
  # HTML colours.
  #
  # Usage examples:
  #
  #   Colours::HtmlColours.does_include? 'slateblue'                 # => true
  #   Colours::HtmlColours.is_this_html_colour_included? 'royalblue' # => true
  #   Colours::HtmlColours.is_this_html_colour_included? 'megawhite' # => false
  #
  # =========================================================================== #
  def self.does_include?(i = :slateblue)
    return true if @hash_html_colours.has_key?(i.to_s)
    false # else return false.
  end; self.instance_eval { alias is_this_html_colour_included? does_include? } # === Colours::HtmlColours.is_this_html_colour_included?
       self.instance_eval { alias is_a_html_colour?             does_include? } # === Colours::HtmlColours.is_a_html_colour?
       self.instance_eval { alias is_a_HTML_colour?             does_include? } # === Colours::HtmlColours.is_a_HTML_colour?
       self.instance_eval { alias is_this_a_html_colour?        does_include? } # === Colours::HtmlColours.is_this_a_html_colour?
       self.instance_eval { alias html_colours_include?         does_include? } # === Colours::HtmlColours.html_colours_include?
       self.instance_eval { alias include_this_html_colour?     does_include? } # === Colours::HtmlColours.include_this_html_colour?
       self.instance_eval { alias is_this_a_valid_html_colour?  does_include? } # === Colours::HtmlColours.is_this_a_valid_html_colour?
       self.instance_eval { alias is_this_input_a_html_colour?  does_include? } # === Colours::HtmlColours.is_this_input_a_html_colour?
       self.instance_eval { alias has_colour?                   does_include? } # === Colours::HtmlColours.has_colour?
       self.instance_eval { alias registered_colour?            does_include? } # === Colours::HtmlColours.registered_colour?

  # =========================================================================== #
  # === Colours::HtmlColours.e
  # =========================================================================== #
  def self.e(i = '')
    ::Colours.e(i)
  end

  # =========================================================================== #
  # === Colours::HtmlColours.available_html_colours?
  #
  # This method will return an Array with all available HTML colours.
  # =========================================================================== #
  def self.available_html_colours?
    @hash_html_colours.keys
  end; self.instance_eval { alias html_colours            available_html_colours? } # === Colours::HtmlColours.html_colours
       self.instance_eval { alias all_html_colours        available_html_colours? } # === Colours::HtmlColours.all_html_colours
       self.instance_eval { alias return_all_html_colours available_html_colours? } # === Colours::HtmlColours.return_all_html_colours
       self.instance_eval { alias all_colours             available_html_colours? } # === HtmlColours.all_colours
       self.instance_eval { alias all_colours?            available_html_colours? } # === HtmlColours.all_colours?
       self.instance_eval { alias colours                 available_html_colours? } # === HtmlColours.colours
       self.instance_eval { alias colours?                available_html_colours? } # === HtmlColours.colours?
       self.instance_eval { alias all?                    available_html_colours? } # === HtmlColours.all?
       self.instance_eval { alias registered_colours?     available_html_colours? } # === HtmlColours.registered_colours?

  # =========================================================================== #
  # === all_html_colours
  #
  # This method will feedback all html colours known.
  # =========================================================================== #
  def all_html_colours
    ::Colours::HtmlColours.available_html_colours?
  end; alias html_colours? all_html_colours # === html_colours?
       alias all_colours   all_html_colours # === all_colours

  begin
    require 'colours/autogenerated/support_for_html_colours.rb'
  rescue LoadError
    puts 'No file exists at colours/autogenerated/support_for_html_colours.rb.'
  end
  # ========================================================================= #
  # === Colours::HtmlColours[]
  #
  # Usage examples:
  #
  #   Colours::HtmlColours['slateblue','Hello World!'] # works as Example 1
  #   Colours::HtmlColours['slateblue']
  #   Colours::HtmlColours['lightgreen','Hello World!']
  #   Colours::HtmlColours[:random]
  #   e Colours::HtmlColours[:random, 'yo there']
  #
  # ========================================================================= #
  def self.[](
      which_colour_to_use, optional_text_to_display = ''
    )
    case which_colour_to_use
    when :random
      which_colour_to_use = random_html_colour
    end
    if ::Colours.use_html_colours?
      ::Colours::SupportForHTMLColours.send(
        which_colour_to_use, optional_text_to_display
      )
    else # else return it unmodified.
      optional_text_to_display
    end
  end

  # ========================================================================= #
  # === Colours::HtmlColours.html_colourize
  # ========================================================================= #
  def self.html_colourize(i)
    if i.is_a? Array
      i = i.join(' ')
    end
    if i
      if File.file?(i)
        i = File.read(i)
      elsif i.respond_to? :read
        i = i.read
      end
    end
    if i.is_a? String
      # ===================================================================== #
      # Find all HTML colours next in that String:
      # ===================================================================== #
      splitted = i.split("\n")
      splitted.map! {|entry|
        if ::Colours.does_this_line_include_a_html_colour?(entry)
          entry = ::Colours.replace_all_raw_html_colours_in_this_line(entry)
        end
        entry
      }
      i = splitted.join("\n")
    end
    e i
  end

  # =========================================================================== #
  # === Colours::HtmlColours.colour_to_rgb('royalblue')
  #
  # This method will convert the long colour name to an Array
  # with R,G,B values.
  #
  # Usage examples:
  #
  #  Colours::HtmlColours.colour_to_rgb('whitesmoke') # => [245, 245, 245]
  #  Colours::HtmlColours.colour_to_rgb('slateblue')  # => [106, 90, 205]
  #  Colours::HtmlColours.colour_to_rgb(:royalblue)   # => [65, 105, 225]
  #
  # =========================================================================== #
  def self.colour_to_rgb(long_name = :slateblue)
    pointer = @hash_html_colours[long_name.to_s] # This will be an Array.
    return [pointer[0], pointer[1], pointer[2]]
  end; self.instance_eval { alias to_rgb colour_to_rgb } # === HtmlColours.to_rgb()

  # =========================================================================== #
  # === Colours::HtmlColours.random_colour?
  #
  # Obtain a random sample, aka a random colour, from all the registered
  # html colours.
  # =========================================================================== #
  def self.random_colour?
    available_html_colours?.sample
  end; self.instance_eval { alias random_colour             random_colour? } # === Colours::HtmlColours.random_colour
       self.instance_eval { alias random                    random_colour? } # === Colours::HtmlColours.random
       self.instance_eval { alias sample                    random_colour? } # === Colours::HtmlColours.sample
       self.instance_eval { alias random_html_colour        random_colour? } # === Colours::HtmlColours.random_html_colour
       self.instance_eval { alias return_random_html_colour random_colour? } # === Colours::HtmlColours.return_random_html_colour
       self.instance_eval { alias return_random_colour      random_colour? } # === Colours::HtmlColours.return_random_colour

end

# =========================================================================== #
# === Colours.show_html_colours
# =========================================================================== #
def self.show_html_colours
  ::Colours::HtmlColours.show_html_colours
end; self.instance_eval { alias display_all_colours show_html_colours } # === Colours.display_all_colours
     self.instance_eval { alias show_colours        show_html_colours } # === Colours.show_colours
     self.instance_eval { alias test                show_html_colours } # === Colours.test
     self.instance_eval { alias show_all_colours    show_html_colours } # === Colours.show_all_colours

# =========================================================================== #
# === show_html_colours
# =========================================================================== #
def show_html_colours
  ::Colours.show_html_colours
end

# =========================================================================== #
# === Colours.random_colour?
#
# Obtain a random sample, aka a random colour, from all the registered
# html colours.
# =========================================================================== #
def self.random_colour?
  ::Colours::HtmlColours.random_colour?
end; self.instance_eval { alias random_colour             random_colour? } # === Colours.random_colour
     self.instance_eval { alias random                    random_colour? } # === Colours.random
     self.instance_eval { alias sample                    random_colour? } # === Colours.sample
     self.instance_eval { alias random_html_colour        random_colour? } # === Colours.random_html_colour
     self.instance_eval { alias return_random_html_colour random_colour? } # === Colours.return_random_html_colour
     self.instance_eval { alias return_random_colour      random_colour? } # === Colours.return_random_colour

# =========================================================================== #
# === Colours.colour_to_rgb
# =========================================================================== #
def self.colour_to_rgb(i = :slateblue)
  ::Colours::HtmlColours.colour_to_rgb(i)
end; self.instance_eval { alias to_rgb colour_to_rgb } # === Colours.to_rgb()

# =========================================================================== #
# === Colours.available_html_colours?
# =========================================================================== #
def self.available_html_colours?
  ::Colours::HtmlColours.available_html_colours?
end; self.instance_eval { alias html_colours?           available_html_colours? } # === Colours.html_colours?
     self.instance_eval { alias html_colours            available_html_colours? } # === Colours.html_colours
     self.instance_eval { alias all_html_colours        available_html_colours? } # === Colours.all_html_colours
     self.instance_eval { alias return_all_html_colours available_html_colours? } # === Colours.return_all_html_colours
     self.instance_eval { alias colours?                available_html_colours? } # === Colours.colours?
     self.instance_eval { alias colours                 available_html_colours? } # === Colours.colours
     self.instance_eval { alias registered_colours?     available_html_colours? } # === Colours.registered_colours?

# =========================================================================== #
# === Colours.hash_html_colours?
# =========================================================================== #
def self.hash_html_colours?
  ::Colours::HtmlColours.hash_html_colours?
end

# =========================================================================== #
# === Colours.does_include?
#
# Usage examples:
#
#   Colours.does_include? 'slateblue'
#   Colours.is_this_html_colour_included? 'royalblue' # => true
#   Colours.is_this_html_colour_included? 'megawhite' # => false
#
# =========================================================================== #
def self.does_include?(i = :slateblue)
  i = i.to_sym unless i.is_a? Symbol
  ::Colours::HtmlColours.does_include?(i)
end; self.instance_eval { alias is_this_html_colour_included? does_include? } # === Colours.is_this_html_colour_included?
     self.instance_eval { alias is_a_html_colour?             does_include? } # === Colours.is_a_html_colour?
     self.instance_eval { alias is_a_HTML_colour?             does_include? } # === Colours.is_a_HTML_colour?
     self.instance_eval { alias is_this_a_html_colour?        does_include? } # === Colours.is_this_a_html_colour?
     self.instance_eval { alias html_colours_include?         does_include? } # === Colours.html_colours_include?
     self.instance_eval { alias include_this_html_colour?     does_include? } # === Colours.include_this_html_colour?
     self.instance_eval { alias is_this_a_valid_html_colour?  does_include? } # === Colours.is_this_a_valid_html_colour?
     self.instance_eval { alias is_this_input_a_html_colour?  does_include? } # === Colours.is_this_input_a_html_colour?
     self.instance_eval { alias has_colour?                   does_include? } # === Colours.has_colour?
     self.instance_eval { alias registered_colour?            does_include? } # === Colours.registered_colour?

# =========================================================================== #
# === available_html_colours?
# =========================================================================== #
def available_html_colours?
   Colours.available_html_colours?
end; alias html_colours?           available_html_colours? # === html_colours?
     alias html_colours            available_html_colours? # === html_colours
     alias all_html_colours        available_html_colours? # === all_html_colours
     alias return_all_html_colours available_html_colours? # === return_all_html_colours
     alias colours?                available_html_colours? # === colours?
     alias colours                 available_html_colours? # === colours
     alias registered_colours?     available_html_colours? # === registered_colours?

# =========================================================================== #
# === Colours.random_html_colour
#
# This method will simply return a (one) random HTML colour, in long
# format, e. g. "teal" or "slateblue".
#
# If you wish to have it colourized properly, you need to use another
# method.
#
# Usage examples:
#
#   Colours.random_colour? # => "teal"
#   Colours.random_colour? # => "blueviolet"
#
# =========================================================================== # 
def self.random_html_colour
  ::Colours::HtmlColours.random_colour?
end; self.instance_eval { alias random_colour?            random_html_colour } # === Colours.random_colour?
     self.instance_eval { alias return_html_colour?       random_html_colour } # === Colours.return_html_colour?
     self.instance_eval { alias random_html_colour?       random_html_colour } # === Colours.random_html_colour?
     self.instance_eval { alias return_random_html_colour random_html_colour } # === Colours.return_random_html_colour

# =========================================================================== #
# === random_colour?
# =========================================================================== #
def random_colour?
  return Colours.random_html_colour
end; alias random_colour             random_colour? # === random_colour
     alias random                    random_colour? # === random
     alias sample                    random_colour? # === sample
     alias random_html_colour        random_colour? # === random_html_colour
     alias return_random_html_colour random_colour? # === return_random_html_colour

# # =========================================================================== #
# # === Colours[]
# # =========================================================================== #
# def self.[](i = :slateblue, optional_text_to_display = '')
#   ::Colours::HtmlColours[i, optional_text_to_display]
# end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  pp Colours::HtmlColours.return_all_html_colours
  p Colours.return_all_html_colours
  pp Colours::HtmlColours.colour_to_rgb('royalblue')
  if ARGV.first
    p Colours::HtmlColours[ARGV.first]
  else
    print 'sienna -> '; pp Colours::HtmlColours.colour_to_rgb(:sienna)
    Colours::HtmlColours.show_all_colours
    p Colours::HtmlColours['sienna']
    e
    Colours::HtmlColours.show_html_colours
    e
  end
  # puts Colours.random_html_colour
  # puts Colours.random_html_colour
  # puts Colours.random_html_colour
  # pp Colours::HtmlColours.available_html_colours?
  # pp Colours::HtmlColours.random_colour?
  pp Colours::HtmlColours.hash_html_colours?
  e
  Colours::HtmlColours.html_colourize(ARGF)
  e
  e 'Random HTML colour: '+Colours.random_html_colour.to_s
  e
  e Colours.random_colour? # => "teal"
  e Colours.random_colour? # => "steelblue"
end