#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file contains code that can be used to autogenerate ruby code,
# and store this into a .rb file. That functionality is mostly useful
# on my home system, though.
# =========================================================================== #
# require 'colours/toplevel_methods/autogenerate.rb'
# =========================================================================== #
module Colours

  require 'yaml'
  require 'colours/constants/newline.rb'
  require 'colours/constants/file_constants.rb'
  require 'colours/toplevel_methods/is_on_roebe.rb'
  require 'colours/256_colours/support_for_256_colours.rb'
  require 'colours/html_colours/html_colours.rb'

  # ========================================================================= #
  # === GENERIC_RUBY_HEADER
  # ========================================================================= #
  GENERIC_RUBY_HEADER = "#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #"

  # ========================================================================= #
  # === Colours.autogenerate_the_module_for_the_html_colours
  #
  # This method will autogenerate the module that will support
  # the html colours.
  # ========================================================================= #
  def self.autogenerate_the_module_for_the_html_colours(
      into = HOME_DIRECTORY_OF_USER_X+'programming/ruby/src/'\
             'colours/lib/colours/autogenerated/support_for_html_colours.rb'
    )
    _ = ''.dup
    _ << GENERIC_RUBY_HEADER+"
# require 'colours/autogenerated/support_for_html_colours.rb'
# =========================================================================== #
"
    _ << "module Colours\n\n"
    _ << "module SupportForHTMLColours # include Colours::SupportForHTMLColours\n\n".dup
    _ << "  require 'colours/toplevel_methods/html_colourize.rb'\n"
    _ << "  require 'colours/toplevel_methods/use_colours.rb'\n\n"
    comment_line = '  # '+('='*73)+' #'+N
    available_html_colours?.each {|name_of_the_html_colour|
      _ << comment_line
      _ << '  # === Colours::SupportForHTMLColours.'+name_of_the_html_colour+N
      _ << comment_line
      _ << "  def self.#{name_of_the_html_colour}(i = '', &block)\n"
      _ << "    if ::Colours.use_html_colours?\n"
      _ << "      return ::Colours.html_colourize(__method__, i, &block)\n"
      _ << "    else\n"
      _ << "      return i\n"
      _ << "    end\n"
      _ << "  end; def #{name_of_the_html_colour}(i = '', &block); SupportForHTMLColours.#{name_of_the_html_colour}(i, &block); end#{N}"
      # ===================================================================== #
      # Next, due to several reasons, we add a prepended "konsole_colour_"
      # as name to this method.
      # ===================================================================== #
      _ << "       def konsole_colour_#{name_of_the_html_colour}(i = '', &block); SupportForHTMLColours.#{name_of_the_html_colour}(i, &block); end#{N}#{N}"
    }
    _ << 'end; end'
    what = _
    # ======================================================================= #
    # This is only useful on my home system really:
    # ======================================================================= #
    puts 'Storing into:'
    puts
    puts '  '+into
    puts
    require 'save_file'
    SaveFile.write_what_into(what, into)
  end

  # ========================================================================= #
  # === Colours.autogenerate_toplevel_basic_colour_methods
  #
  # The code in this method will autogenerate the toplevel colour
  # methods, such as Colours.yellow().
  #
  # The code needs to respect several settings. For example, whether colours
  # are used or not. Additionally, for methods such as Colours.yellow(),
  # we need to find out whether we use the basic colours, the 256-colours or
  # the HTML colours, and call the corresponding method in these cases.
  # ========================================================================= # 
  def self.autogenerate_toplevel_basic_colour_methods(
      into = HOME_DIRECTORY_OF_USER_X+'programming/ruby/src/'\
             'colours/lib/colours/autogenerated/toplevel_basic_colour_methods.rb'
    )
    comment_line = "  # ======================================================================== \n"
    store_this_string = ''.dup
    header = <<-EOF
#{GENERIC_RUBY_HEADER}
# This file contains the code that will call the corresponding colour method
# depending on which colour-mode the user prefers.
# =========================================================================== #
# require 'colours/autogenerated/toplevel_basic_colour_methods.rb'
# =========================================================================== #
module Colours

require 'colours/toplevel_methods/use_colours.rb'
require 'colours/basic_colours/basic_colours.rb'
require 'colours/256_colours/support_for_256_colours.rb'
require 'colours/autogenerated/support_for_html_colours.rb'

module AllColourMethods # include Colours::AllColourMethods

EOF

    store_this_string << header
    # ======================================================================= #
    # First define the methods for the basic colours:
    # ======================================================================= #
    array = return_a_unique_array_containing_all_available_colours
    array.each {|this_basic_colour| # <- e. g. "yellow".
      this_basic_colour = this_basic_colour.dup
      if this_basic_colour.include?(' ') and
         array.include?(this_basic_colour.delete(' '))
        # =================================================================== #
        # === This must be an alias
        #
        # In this case we will also add an alias containing the '_' part.
        # We will only add this as an alias for include-actions, though.
        # (This may change at a later point in the future - not sure yet.)
        # =================================================================== #
        store_this_string << "       def #{this_basic_colour.tr(' ','_')}(i = ''); AllColourMethods.#{this_basic_colour.delete(' ')}(i); end\n"
      else
        name_of_the_method = this_basic_colour.downcase.delete(' ').dup
        store_this_string << comment_line
        store_this_string << "  # === Colours::AllColourMethods.#{name_of_the_method}\n"
        store_this_string << comment_line
        store_this_string << "  def self.#{name_of_the_method}(i = '', &block)\n"
        store_this_string << "    if ::Colours.use_colours?\n"
        # =================================================================== #
        # Ok, so we determine what to do with this colour-method:
        # =================================================================== #
        store_this_string << "      if ::Colours.is_this_a_html_colour?(__method__)\n"
        store_this_string << "        return ::Colours::SupportForHTMLColours.#{name_of_the_method}(i, &block)\n"
        store_this_string << "      elsif ::Colours.is_this_a_256_colour?(__method__)\n"
        store_this_string << "        return ::Colours::SupportFor256Colours.#{name_of_the_method}(i, &block)\n"
        store_this_string << "      else\n"
        store_this_string << "        return ::Colours::BasicColours.#{name_of_the_method}(i, &block)\n"
        store_this_string << "      end\n"
        store_this_string << "    else\n"
        store_this_string << "      return i\n"
        store_this_string << "    end\n"
        store_this_string << "  end; def #{name_of_the_method}(i = '', &block); AllColourMethods.#{name_of_the_method}(i, &block); end\n"
        # =================================================================== #
        # Next, due to several reasons, we add a prepended "konsole_colour_"
        # as name to this method.
        # =================================================================== #
        store_this_string << "       def konsole_colour_#{name_of_the_method}(i = ''); AllColourMethods.#{name_of_the_method}(i); end\n"
        store_this_string << "       def konsole_#{name_of_the_method}(i = ''); AllColourMethods.#{name_of_the_method}(i); end\n"
        store_this_string << "       self.instance_eval { alias konsole_colour_#{name_of_the_method} #{name_of_the_method} } # === Colours::AllColourMethods.konsole_colour_#{name_of_the_method}\n"
        store_this_string << "       def Colours.e#{name_of_the_method}(i = ''); puts AllColourMethods.#{name_of_the_method}(i); end\n"
        store_this_string << "       def e#{name_of_the_method}(i = ''); puts AllColourMethods.#{name_of_the_method}(i); end\n"
        store_this_string << "       alias ekonsole_colour_#{name_of_the_method} e#{name_of_the_method}\n"
        store_this_string << "\n"
      end
    }
    store_this_string << "end; end\n"
    what = store_this_string .dup
    # ======================================================================= #
    # This is only useful on my home system really:
    # ======================================================================= #
    puts 'Storing into the file:'
    puts
    puts "  #{into}"
    puts
    File.delete(into) if File.exist? into # Get rid of the old file.
    begin
      require 'save_file'
      SaveFile.write_what_into(what, into)
    rescue LoadError
      puts 'save_file gem is not available/installed.'
    end
  end; self.instance_eval { alias define_colour_methods autogenerate_toplevel_basic_colour_methods } # === Colours.define_colour_methods

  # ========================================================================= #
  # === Colours.autogenerate_the_module_for_the_256_colours
  #
  # This method will autogenerate the module that will support
  # the 256 colours.
  # ========================================================================= #
  def self.autogenerate_the_module_for_the_256_colours
    _ = ''.dup
    _ << GENERIC_RUBY_HEADER+"
# require 'colours/autogenerated/support_for_256_colours.rb'
# =========================================================================== #
"
    _ << "module Colours\n\n"
    _ << "module SupportFor256Colours # include Colours::SupportFor256Colours\n\n".dup
    _ << "  require 'colours/256_colours/support_for_256_colours.rb'\n\n"
    comment_line = '  # '+('='*73)+' #'+N
    @dataset_for_the_256_colours.each {|name_of_the_colour, id|
      use_this_name = name_of_the_colour.downcase
      _ << comment_line
      _ << '  # === Colours::SupportFor256Colours.'+use_this_name+N
      _ << comment_line
      _ << "  def self.#{use_this_name}(i = '')#{N}"
      _ << "    if Colours.use_colours?\n"
      _ << "      return Colours.return_this_256_colour(__method__, i)\n"
      _ << "    else\n"
      _ << "      return i\n"
      _ << "    end\n"
      _ << "  end; def #{use_this_name}(i = ''); SupportFor256Colours.#{use_this_name}(i); end#{N}#{N}"
    }
    _ << 'end; end'
    what = _
    # ======================================================================= #
    # This is only useful on my home system really:
    # ======================================================================= #
    into = HOME_DIRECTORY_OF_USER_X+'programming/ruby/src/'\
           'colours/lib/colours/autogenerated/support_for_256_colours.rb'
    puts 'Storing into:'
    puts
    puts '  '+into
    puts
    require 'save_file'
    SaveFile.write_what_into(what, into)
  end

  require 'colours/toplevel_methods/html_colourize.rb'
  # ========================================================================= #
  # === Colours.generate_shell_file_containing_the_html_colours
  #
  # This method will generate a shell file into the current directory
  # by default.
  #
  # The code in that file will generate a shell script that holds all
  # the HTML-colours, via UPCASED names.
  #
  # These entries will then look like this:
  #
  #   SADDLEBROWN:  "\e[38;2;139;69;19m"
  #   PERU:         "\e[38;2;205;133;63m"
  #   CHOCOLATE:    "\e[38;2;210;105;30m"
  #
  # ========================================================================= #
  def self.generate_shell_file_containing_the_html_colours(
      generate_the_shell_file_into_this_directory = Dir.pwd
    )
    require 'save_file/module'
    unless generate_the_shell_file_into_this_directory.end_with? '/'
      if generate_the_shell_file_into_this_directory.frozen?
        generate_the_shell_file_into_this_directory = generate_the_shell_file_into_this_directory.dup
      end
      generate_the_shell_file_into_this_directory << '/'
    end
    result = ''.dup
    result << "# This is a shell file that contains the HTML colours\n"
    available_html_colours?.each {|this_colour|
      colour_code_to_use = ::Colours.html_colourize(this_colour.to_sym,'',false).inspect
      result << "export #{this_colour.upcase}=#{colour_code_to_use}\n"
    }
    generate_the_shell_file_into_this_directory << 'shell_file_containing_the_html_colours.sh'
    into = generate_the_shell_file_into_this_directory
    e "Now creating a shell file at `#{into}`."
    SaveFile.write_what_into(result, into)
    # ======================================================================= #
    # Generate files on my home system as well:
    # ======================================================================= #
    if is_on_roebe?
      into = '/home/x/programming/ruby/src/'\
             'roebe/lib/roebe/shell_scripts/'+
             File.basename(generate_the_shell_file_into_this_directory)
      puts "And also saving into `#{into}`."
      SaveFile.write_what_into(result, into)
    end
  end; self.instance_eval { alias autogenerate_shell_file_containing_the_html_colours generate_shell_file_containing_the_html_colours } # === Colours.autogenerate_shell_file_containing_the_html_colours

  # ========================================================================= #
  # === Colours.return_a_unique_array_containing_all_available_colours
  # ========================================================================= #
  def self.return_a_unique_array_containing_all_available_colours
    array = []
    array << YAML.load_file(FILE_HTML_COLOURS).keys                # (1) First the HTML colours
    array << YAML.load_file(FILE_256_COLOURS).keys.map(&:downcase) # (2) Then the basic colours
    array << YAML.load_file(FILE_BASIC_COLOURS)                    # (3) And finally the basic colours
    array.flatten!
    array.uniq!
    array.map!(&:strip)
    array.sort
  end; self.instance_eval { alias all_available_colour_methods? return_a_unique_array_containing_all_available_colours } # === Colours.all_available_colour_methods?

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e
  # pp Colours.return_a_unique_array_containing_all_available_colours
  # e
  Colours.autogenerate_the_module_for_the_html_colours
  Colours.autogenerate_shell_file_containing_the_html_colours
  Colours.autogenerate_the_module_for_the_256_colours
  Colours.autogenerate_toplevel_basic_colour_methods
end # autogeneratecolours