#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/col.rb'
# =========================================================================== #
module Colours

  require 'colours/colour_methods/colour_methods.rb'

  # ========================================================================= #
  # === Colours.col                                                 (col tag)
  #
  # Careful - this method may collide with other methods named col().
  #
  # Usage examples:
  #
  #   Colours.col '/Depot/j/geojgirjh'
  #   Colours.col '/Users/x/AUDIO/'
  #   Colours.col '/Users/x/AUDIO/Westbam_Sunshine.mp3'
  #
  # ========================================================================= #
  def self.col(
      i,
      optional_arg_not_in_use_right_now = ''
    )
    if File.exist? i
      ftype = File.ftype(i)
      # e 'The ftype is: '+ftype # <- This could be used for debugging.
      case ftype
      # ===================================================================== #
      # === file
      # ===================================================================== #
      when 'file'
        return sfile(i)
      # ===================================================================== #
      # === directory
      # ===================================================================== #
      when 'directory'
        return sdir(i)
      # ===================================================================== #
      # === link
      # ===================================================================== #
      when 'link'
        return ssymlink(i)
      else
        e "module Colours: We do not know the filetype `#{ftype}`"
      end
    else # Else return the input a bit changed.
      return sfile(i) # We modify it because that is better.
    end
  end

end