#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/html_colourize.rb'
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/revert.rb'

  # ========================================================================= #
  # === Colours.html_colourize
  #
  # This method will simply return the colour-code + text.
  #
  # The third argument, called `append_revert`, can be used to determine
  # whether we will append the revert code to the generated output String
  # or whether we will not. By default we will, but there are examples
  # where we may wish to assemble our own colour string, and in these
  # cases we do not yet know which text is to be shown - hence, this
  # must become an optional argument.
  #
  # A block can be passed to this method. If the block has, as content,
  # the Symbol :omit_end then the end-part of the ANSI code will not
  # be used.
  #
  # Usage examples:
  #
  #   x = Colours.colourize('slateblue', 'Hello world!'); pp x
  #   y = Colours.colourize('slateblue', 'Hello world!') { :omit_end }; pp y
  #
  # ========================================================================= #
  def self.html_colourize(
      colour_to_use = return_random_html_colour,
      this_text     = nil,
      append_revert = true,
      &block
    )
    require 'colours/html_colours/html_colours.rb'
    require 'colours/rgb/rgb.rb'
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :omit_end
      # ===================================================================== #
      when :omit_end
        append_revert = false
      end
    end
    if this_text.nil?
      this_text = colour_to_use.to_s.dup
    end
    if append_revert
      "#{rgb_value_as_escape_code_string(colour_to_use)}#{this_text}#{revert}"
    else
      "#{rgb_value_as_escape_code_string(colour_to_use)}#{this_text}"
    end
  end; self.instance_eval { alias colourize       html_colourize } # === Colours.colourize
       self.instance_eval { alias kde_colour      html_colourize } # === Colours.kde_colour
       self.instance_eval { alias konsole_colours html_colourize } # === Colours.konsole_colours

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e Colours.html_colourize('slateblue', 'Hello world 1!')
  e Colours.html_colourize(:crimson, 'Hello world 2!')
end