#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/italic.rb'
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/revert.rb'
  require 'colours/toplevel_methods/e.rb'
  require 'colours/rgb/rgb.rb'

  # ========================================================================= #
  # === ITALIC_CODE
  #
  # If you wish to test italic output quickly on the terminal, then you
  # can use something like this:
  #
  #   echo -e "\e[3mone two three\e[23m"
  #
  # ========================================================================= #
  ITALIC_CODE = '3'

  # ========================================================================= #
  # === Colours.return_italic
  #
  # To test this method, try:
  #
  #   Colours.italic('Hello world!')
  #
  # ========================================================================= #
  def self.return_italic(
      show_this_text = 'This must be italic.', 
      make_newline   = false
    )
    result = "\x1b".dup
    result << '['
    result << ITALIC_CODE
    if block_given?
      # ===================================================================== #
      # Right now we assume that this must be a colour.
      # ===================================================================== #
      result << ";38;2;"
      result << html_colour_to_stringified_rgb_values(yield)
    end
    result << 'm'
    if make_newline
      show_this_text = "#{show_this_text}\n"
    end
    result << show_this_text
    result << REVERT
    result
  end; self.instance_eval { alias string_italic return_italic } # === Colours.string_italic

  # ========================================================================= #
  # === Colours.italic
  #
  # This method will "puts" the result of applying italic to a string,
  # on the console/terminal. If you wish to do the output on your own
  # then you have to use the method Colours.return_italic or its
  # alias called Colours.string_italic.
  #
  # To test this, try:
  #   Colours.italic('Hello world!')
  # ========================================================================= #
  def self.italic(
      i            = '', 
      make_newline = false,
      &block
    )
    e return_italic(i, make_newline, &block)
  end

  # ========================================================================= #
  # === italic
  # ========================================================================= #
  def italic(
      i            = '',
      make_newline = false
    )
    ::Colours.italic(i, make_newline)
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  alias ee print
  txt = 'Hello world!'
  if ARGV.empty?
    require 'colours/requires/require_all_colour_methods.rb'
    COLOURS = Colours::AllColourMethods
    system 'clear'
    Colours.italic(txt)
    e COLOURS.palegreen('italic in palegreen:')
    ee '  '; e Colours.string_italic(txt) { :palegreen }
    e COLOURS.slateblue('italic in slateblue:')
    ee '  '; e Colours.string_italic(txt) { :slateblue }
    e COLOURS.orange('italic in orange:')
    ee '  '; e Colours.string_italic(txt) { :orange }
    e COLOURS.crimson('italic in crimson:')
    ee '  '; e Colours.string_italic(txt) { :crimson }
    e 'Italic without colours:'
    ee '  '; e 'Die '+Colours.string_italic('R-Gruppe bei')+
               ' Serin (S) besteht aus ...'
  else
    ee '  '; e Colours.string_italic(ARGV.join(' ').strip)
    ee '  '; e Colours.string_italic(ARGV.join(' ').strip) { :crimson }
  end
end # italic