#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/prefer_this_colour_schemata.rb'
# =========================================================================== #
module Colours

  require 'colours/requires/require_yaml.rb'
  require 'colours/html_colours/add_html_colours_onto_the_toplevel_namespace.rb'
  require 'colours/autogenerated/support_for_html_colours.rb'

  # ========================================================================= #
  # === @prefer_this_colour_schemata
  #
  # The following Array ranks which colour schemata we will try to use.
  # More important entries should come on top.
  #
  # Konsole will be the default, normally, followed by the 256 colours; and
  # then the basic colours.
  #
  # Of course you can decide on your own via the .yml file, and other
  # methods that allow us to change this setting, but when it comes
  # to implementations such as for simp() or sfancy() then we will
  # honour the setting in the yaml file (if we use any colours at all,
  # that is). The yaml file is typically called
  # "prefer_this_colour_schemata.yml".
  # ========================================================================= #
  if File.exist? FILE_PREFER_THIS_COLOUR_SCHEMATA
    @prefer_this_colour_schemata = YAML.load_file(FILE_PREFER_THIS_COLOUR_SCHEMATA)
  end

  # ========================================================================= #
  # === @use_html_colours
  #
  # If the following variable is set to true then the HTML colours will
  # be used, if available. These are colours such as 'slateblue' or
  # 'crimson'.
  # ========================================================================= #
  @use_html_colours = false

  # ========================================================================= #
  # === @use_256_colours
  # ========================================================================= #
  @use_256_colours = false

  # ========================================================================= #
  # === @use_basic_colours
  # ========================================================================= #
  @use_basic_colours = false

  # ========================================================================= #
  # Next, we will set some other toplevel instance variables based on
  # the value of this yaml file.
  # ========================================================================= #
  if @prefer_this_colour_schemata
    first_entry  = @prefer_this_colour_schemata[1]
    _second_entry = @prefer_this_colour_schemata[2]
    _third_entry  = @prefer_this_colour_schemata[3]

    case first_entry
    # === :html_colours
    when :html_colours
      @use_html_colours = true
    # === :support_for_256_colours
    when :support_for_256_colours
      @use_256_colours = true
    # === :basic_colours
    when :basic_colours
      @use_basic_colours = true
    end
  end

  # ========================================================================= #
  # === Colours.prefer_which_colour_schemata?
  # ========================================================================= #
  def self.prefer_which_colour_schemata?
    @prefer_this_colour_schemata[1].to_sym if @prefer_this_colour_schemata
  end; self.instance_eval { alias prefer_which_colour_scheme? prefer_which_colour_schemata? } # === Colours.prefer_which_colour_scheme

  # ========================================================================= #
  # Designate @use_256_colours to true:
  # ========================================================================= #
  if prefer_which_colour_schemata? == :support_for_256_colours
    @use_256_colours = true
  end

end