#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/remove_escape_sequence.rb'
# Colours.remove_escape_sequence
# =========================================================================== #
module Colours

  require 'colours/constants/misc.rb'

  # ========================================================================= #
  # === Colours.remove_escape_sequence
  #
  # The method Colours.remove_escape_sequence() will remove all Ansi 
  # Escape sequences from a given string.
  # ========================================================================= #
  def self.remove_escape_sequence(i = ARGV)
    if i.is_a? Array
      i = i.join("\n")
    end
    i = i.to_s.dup
    # ======================================================================= #
    # Iterate over the registered ansi-colours next.
    # ======================================================================= #
    ARRAY_REGISTERED_ANSI_COLOURS.each {|entry|
      if i.include? entry
        entry = Regexp.quote(entry)
        i.sub!(/#{entry}/, '')
      end
      # ===================================================================== #
      # Next, check for KDE konsole colours. We must use "" there, not ''.
      # ===================================================================== #
      if i.include?("\e[") and i =~ /\d+m/ # Such as: "\e[38;2;220;20;60m|"
        # =================================================================== #
        # The next regex will (hopefully) remove all escape-characters
        # from the given String.
        # =================================================================== #
        regex_for_html_colours = # Detect entries such as: \e[38;2;106;90;205m
          /(\e\[\d{2};\d{1};\d{3};\d{1,2};\d{3}m)/ # See: https://rubular.com/r/tG3XeOK5NPsfmI
        i.gsub!(regex_for_html_colours, '')
        # =================================================================== #
        # See: https://rubular.com/r/SdS28fAGSxIELn
        # =================================================================== #
        # /(\\e\[\d{1,2};?\d{1,2};?\d{0,3};?\d{0,3};?\d{1,2}m|\\e\[\d{0,2}m|\\e\[K|\\e\[\d{0,2};\d{0,2}m|(\\e\[.+\dm)|\\e\[0;37m)/
        [
          /\e\[\d{1,2};?\d{1,2};?\d{0,3};?\d{0,3};?\d{1,2}m/,
          /\\e\[\d{0,2}m|\\e\[K|\\e\[\d{0,2};\d{0,2}m/,
          /\\e\[.+\dm/,
          /\\e\[0;37m/
        ].each {|regex_to_use|
          i.gsub!(regex_to_use, '')
        }
      end
    }
    return i
  end; self.instance_eval { alias remove                  remove_escape_sequence } # === Colours.remove
       self.instance_eval { alias remove_escape_sequences remove_escape_sequence } # === Colours.remove_escape_sequences
       self.instance_eval { alias remove_colours          remove_escape_sequence } # === Colours.remove_colours
       self.instance_eval { alias escape                  remove_escape_sequence } # === Colours.escape

  # ========================================================================= #
  # === Colours.remove_trailing_end_from
  #
  # The second argument to this method can be the escape sequence that you
  # wish to remove.
  #
  # Invocation example:
  #
  #   x = Colours.remove_trailing_end_from("\e[38;2;70;130;180m\e[0;37m") # => "\e[38;2;70;130;180m"
  #
  # ========================================================================= #
  def self.remove_trailing_end_from(
      i,
      remove_this_escape_sequence = revert?
    )
    quoted = Regexp.quote(remove_this_escape_sequence)
    # ======================================================================= #
    # Anchor it at the end via $.
    # ======================================================================= #
    return i.sub(
      /#{quoted}$/, ''
    )
  end; self.instance_eval { alias remove_trailing_escape_part       remove_trailing_end_from } # === Colours.remove_trailing_escape_part
       self.instance_eval { alias remove_trailing_ansii_escape_code remove_trailing_end_from } # === Colours.remove_trailing_ansii_escape_code
       self.instance_eval { alias remove_trailing_ANSII_escape_code remove_trailing_end_from } # === Colours.remove_trailing_ANSII_escape_code
       self.instance_eval { alias remove_trailing_escape_code       remove_trailing_end_from } # === Colours.remove_trailing_escape_code
       self.instance_eval { alias remove_trailing_code              remove_trailing_end_from } # === Colours.remove_trailing_code

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  begin
    require 'colours/autoinclude'
  rescue LoadError; e 'the colours gem is not available.'; end
  include Colours
  e
  e 'Next testing an escape sequence (which will be shown via pp only):'
  e
  x = "\e[01m\e[Kcc1:\e[m\e[K \e[01;35m\e[Kwarning: \e[m\e[K-Wabi won't warn about anything [\e[01;35m\e[K-Wabi\e[m\e[K]

A\e[38;2;220;20;60m|AGCTT"
  pp x
  e
  x = Colours.remove_escape_sequence(x)
  e
  pp x
  e x
  pp Colours.remove_escape_sequences(Colours.slateblue('Hello world!')) # => "\e[38;2;106;90;205mHello world!"
  p Colours.slateblue('Hello world!') # => "\e[38;2;106;90;205mHello world!\e[0;37m"
end