#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal =  true
# =========================================================================== #
# require 'colours/toplevel_methods/revert.rb'
# =========================================================================== #
module Colours

  require 'colours/constants/escape.rb'

  # ========================================================================= #
  # === Colours::COLOUR_NC
  #
  # Note that this will actually reset all attributes.
  #
  # The "\e[0m" sequence removes all attributes, including formatting
  # and colours. It may be useful to add it to the end of each colour
  # text - and this is what the Colours project is essentially doing.
  #
  # An alternative may be to assign REVERT to WHITE, but this seems
  # to be incorrect according to the specification.
  # ========================================================================= #
  COLOUR_NC        = "#{ESC}0m" # No Colour.
    REVERT         = COLOUR_NC    # === REVERT
    REV            = COLOUR_NC    # === REV
    CLOSING_TAG    = COLOUR_NC    # === CLOSING_TAG (also represented as "#{ESC}0m")
    RESET          = COLOUR_NC    # === RESET
    RESET_TERMINAL = COLOUR_NC    # === RESET_TERMINAL
    CLEAR          = COLOUR_NC    # === CLEAR
    COLOUR_RESET   = COLOUR_NC    # === COLOUR_RESET
    DEFAULT_COLOUR = COLOUR_RESET # === DEFAULT_COLOUR

  # ========================================================================= #
  # === RESET_ALL_ATTRIBUTES
  #
  # This is essentially the same as COLOUR_NC / RESET.
  # ========================================================================= #
  RESET_ALL_ATTRIBUTES = "\x1b[0m"

  # ========================================================================= #
  # === @revert
  #
  # We need a variable here in order to be able to change this.
  # ========================================================================= #
  @revert = "\e[0;37m"

  # ========================================================================= #
  # === Colours.set_revert
  # ========================================================================= #
  def self.set_revert(i = :default)
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = REVERT
    end
    @revert = i
  end

  # ========================================================================= #
  # === Colours.rev
  #
  # We use this method to revert to the old default, that is, the prior
  # state for ANSI colours. It should be equivalent to: "\x1b[0m" e. g.
  # REVERT = "\x1b[0m" # "\x1b[39;49m" <--- This here seems to fail for Konsole.underline, though.
  #
  # An alternative could be DEFAULT_COLOUR.
  # ========================================================================= #
  def self.rev
    # REVERT # <- This used to be: "\e[0;37m"
    @revert # This is light white. I prefer this colour actually.
  end; self.instance_eval { alias revert   rev } # === Colours.revert
       self.instance_eval { alias revert?  rev } # === Colours.revert?
       self.instance_eval { alias restore? rev } # === Colours.restore?

  # ========================================================================= #
  # === rev
  #
  # Use this method to "revert" to the old default colour again, by
  # delegating towards the Colours.rev() method defined above.
  #
  # From this point on, we use methods that can be toggled.
  # ========================================================================= #
  def rev(i = '')
    ::Colours.rev # This will refer to the Colours.rev() module method.
  end; alias revert rev # === revert
       # alias default_colour rev

  # ========================================================================= #
  # === restore?
  #
  # This will restore to the default again.
  # ========================================================================= #
  def restore?
    Colours.restore?
  end; alias revert restore? # === revert
       alias rev?   restore? # === rev?

end

if __FILE__ == $PROGRAM_NAME
  pp Colours.rev
end