#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will handline underline-functionality in a terminal. It
# supports basic colours and the HTML colours.
# =========================================================================== #
# require 'colours/toplevel_methods/underline.rb'
# Colours.return_underline
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/revert.rb'
  require 'colours/toplevel_methods/e.rb'
  require 'colours/rgb/rgb.rb'

  # ========================================================================= #
  # === UNDERLINE_CODE
  #
  # To quickly test the underline-functionality, you can try:
  #
  #   echo -e "\e[4mone two three\e[23m" 
  #
  # ========================================================================= #
  UNDERLINE_CODE = '4'

  # ========================================================================= #
  # === Colours.underline
  #
  # This method will "puts" the result of applying underline to a string,
  # on the console/terminal. If you wish to do the output on your own
  # then you have to use the method Colours.return_underline or its
  # alias called Colours.string_underline.
  #
  # To test this, try:
  #
  #   Colours.underline('Hello world!')
  #
  # ========================================================================= #
  def self.underline(
      i            = '', 
      make_newline = false,
      &block
    )
    e return_underline(i, make_newline, &block)
  end

  # ========================================================================= #
  # === underline
  # ========================================================================= #
  def underline(
      i            = '',
      make_newline = false
    )
    Colours.underline(i, make_newline)
  end

  # ========================================================================= #
  # === Colours.return_underline
  #
  # The point of this method here is to make a given text (String) appear
  # "underlined", via ANSI escape sequences. For underline the String
  # that should be used is "\u001b[4m".
  #
  # This can be tested on the commandline such as via:
  #
  #   echoen "\u001b[4m Underline \u001b[0m"
  #
  # Usage examples:
  #
  #   puts ' ok | '+Colours.return_underline('Hello world!')+' | ok'
  #   puts ' ok | '+Colours.return_underline('Hello world!') { :slateblue }+' | ok'
  #
  # In January 2022 the second variant was removed, though. Let's keep things
  # simple for now - perhaps in the future this may be re-enabled.
  # ========================================================================= #
  def self.return_underline(
      show_this_text          = 'This must be underline.',
      make_newline            = false,
      use_this_as_revert_code = REVERT
    )
    # ===================================================================== #
    # Build up our main string that codes for underline.
    # ===================================================================== #
    result = "\u001b[#{UNDERLINE_CODE}m".dup
    # if block_given?
    #   # ===================================================================== #
    #   # Right now we assume that this must be a colour if it is supplied
    #   # via a block. However had, in January 2022 I realised that this
    #   # assumption is not always correct, so this was disabled for now.
    #   # At a later moment in time we may reconsider this.
    #   # ===================================================================== #
    #   result << ";38;2;"
    #   colour_replacement = colour_to_rgb_value(yield)
    #   result << colour_replacement
    # end
    # result << 'm'
    if make_newline # Append a newline in this case.
      show_this_text = "#{show_this_text}\n"
    end
    result << show_this_text
    result << use_this_as_revert_code
    return result
  end; self.instance_eval { alias string_underline return_underline } # === Colours.string_underline

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  if ARGV.empty?
    e Colours.underline('Hello world!')
    e 'Hello world!'
    # ======================================================================= #
    # And then run some more tests:
    # ======================================================================= #
    require 'colours/toplevel_methods/use_colours.rb'
    Colours.enable_html_colours
    e Colours.palegreen('in palegreen:')
    print '  '; print Colours.underline(txt) { :palegreen }
    e Colours.slateblue('in slateblue:')
    print '  '; print Colours.underline(txt) { :slateblue }
    e Colours.orange('in orange:')
    print '  '; print Colours.underline(txt) { :orange }
    e Colours.crimson('in crimson:')
    print '  '; print Colours.underline(txt) { :crimson }
  else
    puts Colours.underline(ARGV.join(' ').strip)
    puts Colours.underline(ARGV.join(' ').strip) { :crimson }
  end
end # underline one TWO three